
=head1 NAME

Device::Osram::Lightify - The version of our release.

=head1 DESCRIPTION

This module just stores the version of our code.

=cut

=head1 SYNOPSIS

   use Device::Osram::Version;

   print $Device::Osram::Lightify::VERSION;

=cut

=head1 AUTHOR

Steve Kemp <steve@steve.org.uk>

=cut

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2016 Steve Kemp <steve@steve.org.uk>.

This library is free software. You can modify and or distribute it under
the same terms as Perl itself.

=cut

package Device::Osram::Lightify;

use strict;
use warnings;

our $VERSION = "0.7.1";

1;
